import edu.neu.ccs.*;
import edu.neu.ccs.gui.*;
import edu.neu.ccs.util.*;
import edu.neu.ccs.filter.*;
import edu.neu.ccs.console.*;
import edu.neu.ccs.pedagogy.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.util.*;
import javax.swing.*;
import java.lang.reflect.*;

public class HanoiApplication extends DisplayPanel
    implements JPTConstants, ConsoleAware
{
    
    ////////////////////
    // Model and View //
    ////////////////////
    
    /** The Hanoi model. */
    protected final HanoiModel model = new HanoiModel();
    
    /** The Hanoi view. */
    protected final HanoiView view = model.getHanoiView();
    
    //////////////////
    // Delay Slider //
    //////////////////
    
    /** Slider minimum value. */
    public static int SLIDER_MINIMUM_VALUE = 10;
    
    /** Slider minimum on view. */
    protected static int SLIDER_MINIMUM = 0;
    
    /** Slider maximum on view. */
    protected static int SLIDER_MAXIMUM = 200;
    
    /** Slider ticks on view. */
    protected static int SLIDER_TICKS = 20;
    
    /** Delay slider header prompt in GUI. */
    protected static String ANIMATION_DELAY = "Animation Delay";
    
    /** Delay slider value prompt in GUI. */
    protected static String DELAY = "Delay = ";
    
    /** The delay slider. */
    protected SliderView delaySlider =
        new SliderView
            (HORIZONTAL, SLIDER_MINIMUM, SLIDER_MAXIMUM, SLIDER_MINIMUM_VALUE);
    
    /** The delay slider annotation. */
    protected Annotation delaySliderAnnotation =
        new Annotation(DELAY + delaySlider.getValue());
    
    /** The slider sliding action. */
    protected SimpleAction slidingAction =
        new SimpleAction("Sliding Action") {
            public void perform() { slidingAction(); }
        };
    
    /** The slider panel. */
    TablePanel sliderPanel = new TablePanel(
        new Object[] { ANIMATION_DELAY, delaySlider, delaySliderAnnotation },
        HORIZONTAL, 20, 20, CENTER);
    
    //////////////////////
    // Hanoi Disk Panel //
    //////////////////////
    
    protected static String disksPrompt = "Set The Number of Disks <= 8";
    
    protected TextFieldView disksTFV = new TextFieldView("" + HanoiModel.maxDisks, 30);
    
    protected OptionsView disksOptions = new OptionsView(
        new String[] {
            "Place Disks on Tower 0",
            "Place Disks Randomly"    },
        0, new TableLayout(1, 2, 20, 20, CENTER));
    
    protected SimpleAction setDisks = new SimpleAction(disksPrompt) {
        public void perform() { setDisks(); }
    };
    
    protected TablePanel disksPanel = new TablePanel(
        new Object[] { setDisks, disksTFV, disksOptions },
        HORIZONTAL, 20, 20, CENTER);
    
    /////////////////
    // Constructor //
    /////////////////
    
    /** The constructor. */
    public HanoiApplication() {
        makeGUI();
        
        setDisks();
    }
    
    /** Make the six actions for moves of one disk. */
    protected Action[] makeMoveActions() {
        Action[] actions = new Action[6];
        int index = 0;
        
        for (int tower1 = 0; tower1 <= 2; tower1++)
            for (int tower2 = 0; tower2 <= 2; tower2++)
                if (tower1 != tower2) {
                    final int t1 = tower1;
                    final int t2 = tower2;
                    final String name = "Move disk: " + t1 + " to " + t2;
                    
                    actions[index] = new SimpleAction(name) {
                            public void perform() {
                                model.moveDisk(t1, t2);
                            }
                        };
                    
                    index++;
                }
        
        return actions;
    }

    /** Add features to the slider. */
    protected void prepareSlider() {
        delaySlider.setMajorTickSpacing(SLIDER_TICKS);
        delaySlider.installStandardLabels();
        
        Dimension size = delaySlider.getPreferredSize();
        Dimension newSize =
            new Dimension
                (2 * (int) size.getWidth(), (int) size.getHeight());
        delaySlider.setPreferredSize(newSize);
        
        delaySlider.addSlidingAction(slidingAction);
        delaySlider.addReleaseAction(slidingAction);
        
        view.setDelaySlider(delaySlider);
    }
    
    /** The delay slider sliding action method. */
    protected void slidingAction() {
        int value = delaySlider.getValue();
        
        if (value < SLIDER_MINIMUM_VALUE) {
            value = SLIDER_MINIMUM_VALUE;
            delaySlider.setValue(value);
        }
        
        delaySliderAnnotation.setText(DELAY + value);
    }
    
    /** The method to set the number of disks. */
    protected void setDisks() {
        int disks = disksTFV.demandInt();
        
        if (disks < 1)
            disks = 1;
        else
        if (disks > HanoiModel.maxDisks)
            disks = HanoiModel.maxDisks;
        
        disksTFV.setViewState("" + disks);
        
        if (disksOptions.getSelectedIndex() == 0)
            model.initializeTower(0, disks);
        else
            model.randomHanoi(disks);
    }
    
    /** Make the Hanoi application GUI. */
    protected void makeGUI() {
        ActionsPanel actionsPanel = new ActionsPanel(makeMoveActions());
        
        prepareSlider();
        
        disksTFV.addActionListener(setDisks);
        
        TablePanel panel = new TablePanel(
            new Object[] { view, disksPanel, actionsPanel, sliderPanel },
            VERTICAL, 5, 5, NORTH);
        
        add(panel);
    }
    
    /** main */
    public static void main(String[] args) {
        JPTFrame.createQuickJPTFrame("Hanoi", new HanoiApplication());
    }
}